clear;
clc;

% Note: It is necessary to have AToM in MATLAB path.

%% 1/ Definition of the structure to be analyzed (here - meanderline antenna)
L  = 1;     % width of the bounding box (rectangular plate)
H  = 0.5*L; % height of the bounding box (rectangular plate)
s  = L/33;  % width of the gap between strips
w  = s;     % width of the strip
w0 = L/50;  % width of the feeding edge (delta gap), taper is added if needed
D  = w;     % maximum length of triangle edge (for discretization)
a  = hypot(L, H)/2; % radius of the smallest circumscribing sphere

ka = 1;     % electrical size
k  = ka/a;  % wavenumber (in vacuum)

%% 2/ Generation of the polygon representing manderline antenna
[Poly, Nmeanders] = generateMeanders(L, H, s, w, w0);

% Plot generated meanderline antenna inside the bounding box
figure('color', 'w', 'pos', [50 50 1200 600]);
fill([-L; L; L; -L]/2, [-H; -H; H; H]/2, 'k', 'FaceAlpha', 0.1);
hold on;
fill(Poly.x, Poly.y, 'r', 'LineWidth', 2, 'FaceAlpha', 0.4);
text(Poly.x, Poly.y, num2cell(1:length(Poly.x)), 'Color', 'b', ...
    'HorizontalAlignment', 'center', 'FontSize', 12);
xlabel('x');
ylabel('y');
xlim(0.55*[-L L]);
axis equal;

%% 3/ Delaunay discretization with AToM mesher, construction of basis function
% Get Delaunay
Mesh = discretizeMeander(Poly, D);
% Get basis functions
BF = models.solvers.MoM2D.basisFcns.getBasisFcns(Mesh);

% Plot discretized structure
hndl = results.plotMesh(Mesh);
hndl = results.plotBasisFcns(Mesh, BF, 'handles', hndl);

%% 4/ Determination of feeding position, construction of delta gap feeder
% Find triangles to be fed
PtGap  = find(Mesh.nodes(:, 1) == 0); % find all the points with X == 0
TrGap  = find(sum(ismember(Mesh.connectivityList, repmat(PtGap, 3, 1)), 2) == 2);
% Find all triangles containing at least one vertex lying in YZ plane
Tr0_xC = Mesh.triangleCentroids(TrGap, 1);
% Sort out "positive" and "negative" triangles (important for voltage polarity)
Tr_plus  = TrGap(Tr0_xC > 0);
Tr_minus = TrGap(Tr0_xC < 0);

% Standalone definition of voltage delta gap feeder
% 4a/ Definition of standalone delta gaps
gaps.DeltaGap = struct(...
    'number', 1, ...
    'feederClass', 'models.feeding.DeltaGap', ...
    'feederValue', 1, ...
    'feederType', 'voltage', ...
    'portClass', 'models.feeding.GeomDiscretePort2D', ...
    'elements', [Tr_plus Tr_minus]);

% 4b/ Processing of delta gaps considering basis function size and orientation
Gap = models.solvers.MoM2D.excitation.processPorts(Mesh, BF, gaps);
% 4c/ Final calculation of excitation ccefficients
V = models.solvers.MoM2D.computation.vVec(Mesh, BF, Gap, [], k, 3); 

%% 5/ Determination of the resonant frequency
% Estimate frequency for the Newton's method
c0 = models.utilities.constants.c0;
lambdaGeom_over_2 = (L + 2*Nmeanders*H);
f0_start = c0 / lambdaGeom_over_2;

options.Rs    = 0.01;
options.Nquad = 4;

getRealValuedZin = @(x) findResonance(Mesh, BF, V, x, options);
f0 = newton(f0_start, (1+1e-3)*f0_start, getRealValuedZin, 30, 1e-5, @(y) abs(y));

%% 6/ Evaluation of antenna metrics (input impedance, Q-factor, ...)
OP = evaluateOperators(Mesh, BF, f0, options); % evaluation of operators

I   = (OP.L + OP.Z) \ V;  % evaluation of current density
IZI = I'*(OP.L + OP.Z)*I; % complex power
Xin2Rin = imag(IZI) / real(IZI) % how close is the antenna to resonance

% stored energy operators
OP.omXm = (OP.omW + imag(OP.Z))/2;
OP.omXe = (OP.omW - imag(OP.Z))/2;

Prad   = real(I'*real(OP.Z)*I) / 2; % radiated power
Plost  = real(I'*OP.L*I) / 2;       % ohmic lost power

omWM   = real(I'*OP.omXm*I) / 4; % stored magnetic energy
omWE   = real(I'*OP.omXe*I) / 4; % stored electric energy

D      = real(I'*4*pi*OP.U*I) / Prad % antenna directivity
Qmax   = 2*max(omWM, omWE) / Prad    % radiation Q-factor

delta  = Plost / Prad;         % ohmic loss dissipation factor
eta    = Prad / (Prad + Plost) % radiation efficiency

% References:
% ------------------------------------------------------------------------------
% For fundamental bounds on Q-factor and its feasibility with meanderline:
%   [1] Capek, M., Jelinek, L., Schab, K., Gustafsson, M., Jonsson, B.L.G.,
%       Ferrero, F., Ehrenborg, C.: Optimal Planar Electric Dipole Antenna, 
%       IEEE MAP
% For determination of fundamental bounds on Q-factor:
%   [2] Capek, M., Gustafsson, M., Schab, K.: Minimization of Antenna Quality
%       Factor, IEEE TAP
% To download AToM:
%   [3] http://antennatoolbox.com/
% For detailed information about numerical method used and delta gap feeding
%   [4] Harrington, R.: Field Computation by Method of Moments, IEEE-Wiley, 1993
%   [5] Balanis, C. A.: Advanced Engineering Electromagnetics, 1989